{% extends "horizon/common/_modal_form.html" %}
{% load i18n %}

{% block form_id %}edit_alarm_form{% endblock %}
{% block form_action %}{{ action_url }}{% endblock %}

{% block modal-header %}{% trans "Edit Alarm Definition" %}{% endblock %}

{% block modal-body %}
    <h3>{% trans "Description" %}:</h3>
        <p>{% blocktrans %}
          The Name field is used to identify the alarm definition.
        {% endblocktrans %}</p>
    <p>{% blocktrans %}
      The Expression field  which if true, triggers a notification to be sent.
      See <a href="https://github.com/openstack/monasca-api/blob/master/docs/monasca-api-spec.md#alarm-definition-expressions" target="_blank">Alarm Expressions</a> for how to write an expression.
    {% endblocktrans %}</p>
    <p>{% blocktrans %}
      The Notifications field contains the list of Notifications that should be sent when transitioning to another ALARM state.
    {% endblocktrans %}</p>
    <fieldset>
    {% include "horizon/common/_form_fields.html" %}
    </fieldset>
{% endblock %}

{% block modal-footer %}
  <a href="{{ cancel_url }}" class="btn btn-default cancel">{% trans "Cancel" %}</a>
  <input class="btn btn-primary" type="submit" value="{% trans "Save" %}" />
{% endblock %}
