{% load i18n %}

{% block help_message %}
<div class="clearfix">
    <p class="text-info">
        {% blocktrans %}
         Each alarm definition is defined by its expression composed out of
         mathematical function, metric, time, times ,threshold and comparator for
         metric's value and the threshold. Time is the number of seconds for the measurement
         to be done. They can only be in a multiple of 60. Times is how many times in a row
         that expression must be true before triggering the alarm. Both time and times are
         optional and default to 60 and 1 respectively. Additionally it is possible to narrow
         evaluation of the alarm to certain entities by choosing their
         dimensions. The deterministic alarms never enter UNDETERMINED state.
         Use them for metrics that are received sporadically.
        {% endblocktrans %}
    </p>
</div>
{% endblock %}
